<?php

namespace fazacrmclient;

/**
 * Client REST pour fazaCRM
 * @author lpichoud
 */
class fazaCRMRESTClient {

    public $crm_uri = null;
    public $log_path = null;
    public $client = null;
    public $apiKey = null;
    public $apiVersion = null;
    public $message = '';

    public function __construct($crm_uri, $apiVersion, $apiKey) {

        $this->crm_uri = $crm_uri;
        $this->apiVersion = $apiVersion;
        $this->apiKey = $apiKey;

        if ($this->crm_uri === false) {
            $this->message = 'Veuillez configurer l\'adresse du FazaCRM.';
            throw new Exception("Veuillez configurer l'adresse du FazaCRM.");
        }

        if ($this->apiKey !== false) {
            $this->message = 'Clé trouvée passage en version 1.1';
            $this->apiVersion = 1.1;
        }

        $this->logInit();
    }

    /**
     * Retourne la valeur de la propriété message.
     * @return string 
     */
    public function getMessage() {
        return $this->message;
    }

    /**
     * Initialise et créer un dossier de log et stock son chemin
     * dans la propriétée log_path 
     */
    public function logInit() {
        if ($this->log_path === null) {
            $logDir = \fazae\Core\Lib\Context::getLogDir();
            if (!file_exists($logDir)) {
                mkdir($logDir);
            }
            $this->log_path = $logDir . "/fazaCRMRESTClient.log";
        }
    }

    /**
     * Ecrit dans le fichier log défénis par la propriété log_path
     * @param string $message
     */
    function log($message) {
        Log::logToFile($this->log_path, date('Y-m-d H:i:s') . " - " . $message . "\n");
    }

    /**
     * Vérifie la validité de l'URL du fazaCRM (si l'adresse saisie correspond à une IP existante) 
     * @return bool TRUE si l'URL existe et FALSE dans le cas contraire
     */
    function isValidUrl($url) {
        $url = parse_url($url);
        if (!isset($url["host"])) {
            return false;
        }
        return !(gethostbyname($url["host"]) == $url["host"]);
    }

    /**
     * instancie le client HTTP Zend
     * @param type $service
     * @return \Zend\Http\Client
     */
    public function zendInit($service) {
        if (!$this->isValidUrl($this->crm_uri)) {
            $this->client = null;
            $this->log("L'url " . $this->crm_uri . " n'est pas valide, echec de connexion.");
            return false;
        }

        if ($this->client !== null) {
            /* @var $this->client \Zend\Http\Client */
            if ($this->client->getUri() != $this->crm_uri . $service) {
                $this->client->setUri($this->crm_uri . $service);
                $this->log("Changement d'URI - " . $this->crm_uri . $service);
            }
        } else {

            $adapter = new \Zend\Http\Client\Adapter\Curl();
            $adapter = $adapter->setCurlOption(CURLOPT_SSL_VERIFYHOST, false);
            $adapter = $adapter->setCurlOption(CURLOPT_SSL_VERIFYPEER, false);

            $this->client = new \Zend\Http\Client($this->crm_uri . $service);
            $this->client->setAdapter($adapter);
            $this->client->setOptions(array(
                "maxredirects" => 0
            ));
            $this->log('Initialisation du client HTTP OK - ' . $this->crm_uri . $service);
        }
        if ($this->apiVersion >= 1.1) {
            $this->client->setHeaders(['accept' => 'application/json',
                'fazacrm-version' => $this->apiVersion,
                'fazacrm-key' => $this->apiKey]);
        } else {
            $this->client->setHeaders(['accept' => 'application/json']);
        }
    }

    /**
     * création d'un contact
     * @param array $contact_data
     * @return int|bool L'ID du contact s'il est crée/mis à jour et FALSE si une erreur se produit
     */
    public function contactCreate($contact_data) {
        $this->zendInit("contacts");
        if ($this->client === null) {
            return;
        }
        $this->client->setMethod("PUT");
        $this->client->setParameterPost($contact_data);
        $this->log('contactCreate - send');
        try {
            $this->client->send();
        } catch (\Exception $exc) {
            $this->logError("contactCreate - " . $exc->getMessage());
        }

        $response = $this->client->getResponse();
        $content = $this->client->getResponse()->getContent();
        $json_content = json_decode($content, true);

        switch ($response->getStatusCode()) {
            case 201: //\Symfony\Component\HttpFoundation\Response::HTTP_CREATED
                $this->log('contactCreate - réponse HTTP CREATED');
                if (isset($json_content['id'])) {
                    $this->message = 'The contact record has been created !';
                    $this->log('contactCreate - FIN');
                    return $json_content['id'];
                }
                break;
            //Si l'email entrée est non valide.
            case 400: //\Symfony\Component\HttpFoundation\Response::HTTP_BAD_REQUEST
                $this->log('contactCreate - réponse HTTP BAD REQUEST');
                $json_content = json_decode($content, true);
                $this->message = ' is not a valid email.';
                return false;
                break;
            //Si l'accès à l'API nous est refusé parceque la clé n'est pas valide
            case 403: //\Symfony\Component\HttpFoundation\Response::HTTP_FORBIDDEN
                $this->log('contactCreate - réponse HTTP FORBIDEN');
                $this->message = 'Access denied to the FazaCRM\'s API, please check your private key.';
                return false;
                break;
            //conflit, l'entité existe déjà
            case 409: //\Symfony\Component\HttpFoundation\Response::HTTP_CONFLICT
                $this->log('contactCreate - réponse HTTP CONFLICT');
                $json_content = json_decode($content, true);
                if (isset($json_content['id'])) {
                    return $this->contactUpdate($json_content['id'], $contact_data);
                }
                break;
            //Si aucune valeur n'est envoyée lors de l'appel à l'API.
            case 500: //\Symfony\Component\HttpFoundation\Response::HTTP_INTERNAL_SERVER_ERROR
                $this->log('contactCreate - réponse INTERNAL SERVER ERROR');
                $json_content = json_decode($content, true);
                $this->message = 'You send empty data.';
                return false;
                break;
            default:
                $this->message = 'Unknown error occured during the creation of the contact record.';
                $this->logError("contactCreate");
                return false;
                break;
        }
    }

    /**
     * mise à jour d'un contact
     * @param int $contact_id
     * @param int $contact_data
     * @return int|bool L'ID du contact s'il est mis à jour et FALSE si une erreur se produit
     */
    public function contactUpdate($contact_id, $contact_data) {
        $this->zendInit("contacts/" . $contact_id);
        if ($this->client === null) {
            return;
        }

        $this->client->setMethod("POST");
        $this->client->setParameterPost($contact_data);
        $this->log('contactUpdate - send');
        try {
            $this->client->send();
        } catch (\Exception $exc) {
            $this->logError("contactUpdate - " . $exc->getMessage());
        }

        $response = $this->client->getResponse();
        $content = $this->client->getResponse()->getContent();
        $json_content = json_decode($content, true);

        switch ($response->getStatusCode()) {
            case 200: //\Symfony\Component\HttpFoundation\Response::HTTP_OK:
                $this->log('contactUpdate - réponse HTTP OK');
                if (isset($json_content['id'])) {
                    $this->message = 'The contact record has been updated !';
                    $this->log('contactUpdate - FIN');
                    return $json_content['id'];
                }
                return;
                break;
            //Si l'accès à l'API nous est refusé parceque la version est supérieur ou égale à 1.1 et 
            //que la clé n'est pas valide
            case 403: //\Symfony\Component\HttpFoundation\Response::HTTP_FORBIDDEN:
                $this->log('contactUpdate - réponse HTTP FORBIDEN');
                $this->message = 'Access denied to the FazaCRM\'s API, please check your private key.';
                return false;
                break;
            default:
                $this->message = 'Unknown error occured during the update of the contact record.';
                $this->logError("contactUpdate");
                return false;
                break;
        }
    }

    /**
     * création d'un tag
     * @param string $nom
     */
    public function tagCreate($nom) {
        $this->zendInit("tags");
        if ($this->client === null) {
            return;
        }

        $this->client->setMethod("PUT");
        $this->client->setParameterPost(array('nom' => $nom));
        $this->log('tagCreate - send');
        try {
            $this->client->send();
        } catch (\Exception $exc) {
            $this->logError("tagCreate - " . $exc->getMessage());
        }

        $response = $this->client->getResponse();
        $content = $this->client->getResponse()->getContent();
        $json_content = json_decode($content, true);

        switch ($response->getStatusCode()) {
            case 201: //\Symfony\Component\HttpFoundation\Response::HTTP_CREATED:
                $this->message = 'Your tag have been created !';
                $this->log('tagCreate - réponse HTTP CREATED');
                if (isset($json_content['id'])) {
                    $this->log('tagCreate - FIN');
                    return $json_content['id'];
                }
                break;
            //conflit, l'entité existe déjà
            case 409: //\Symfony\Component\HttpFoundation\Response::HTTP_CONFLICT:
                $this->message = 'Your tag already exist.';
                $this->log('tagCreate - réponse HTTP CONFLICT');
                $json_content = json_decode($content, true);
                if (isset($json_content['id'])) {
                    $this->log('tagCreate - FIN');
                    return $json_content['id'];
                }
                break;
            //Si l'accès à l'API nous est refusé parceque la version est supérieur ou égale à 1.1 et 
            //que la clé n'est pas valide
            case 403: //\Symfony\Component\HttpFoundation\Response::HTTP_FORBIDDEN:
                $this->message = 'Access denied to the FazaCRM\'s API, please check your private key.';
                $this->log('tagCreate - réponse HTTP FORBIDEN');
                return false;
                break;
            default:
                $this->message = 'Unknown error occured during the tag creation.';
                $this->logError("tagCreate");
                return false;
                break;
        }
    }

    /**
     * Ajout d'un tag à un contact
     * @param int $contact_id
     * @param int $tag_id
     * @return boolean 
     */
    public function contactTagAdd($contact_id, $tag_id) {
        $this->zendInit("contacts/" . $contact_id . "/tags/" . $tag_id);
        if ($this->client === null) {
            return;
        }

        $this->client->setMethod("POST");
        $this->log('contactTagAdd - send');
        try {
            $this->client->send();
        } catch (\Exception $exc) {
            $this->logError("contactTagAdd - " . $exc->getMessage());
        }

        $response = $this->client->getResponse();
        $content = $this->client->getResponse()->getContent();
        $json_content = json_decode($content, true);
        switch ($response->getStatusCode()) {
            case 201: //\Symfony\Component\HttpFoundation\Response::HTTP_CREATED:
            case 403: //\Symfony\Component\HttpFoundation\Response::HTTP_CONFLICT:
                $this->message = 'The tag have been added to the contact record.';
                $this->log('contactTagAdd - réponse OK');
                $this->log('contactTagAdd - FIN');
                return true;
                break;
            //Si l'accès à l'API nous est refusé parceque la version est supérieur ou égale à 1.1 et 
            //que la clé n'est pas valide
            case 403: //\Symfony\Component\HttpFoundation\Response::HTTP_FORBIDDEN:
                $this->message = 'Access denied to the FazaCRM\'s API, please check your private key.';
                $this->log('contactTagAdd - réponse HTTP FORBIDEN');
                return false;
                break;
            default:
                $this->message = 'Unknown error occured during the tag affectation to this contact record.';
                $this->logError("contactTagAdd");
                return false;
                break;
        }
    }

    /**
     * Suppression d'un tag associé à un contact
     * @param int $contact_id
     * @param int $tag_id
     * @return boolean
     */
    public function contactTagDelete($contact_id, $tag_id) {
        $this->zendInit("contacts/" . $contact_id . "/tags/" . $tag_id);
        if ($this->client === null) {
            return;
        }

        $this->client->setMethod("DELETE");
        $this->log('contactTagDelete - send');
        try {
            $this->client->send();
        } catch (\Exception $exc) {
            $this->logError("contactTagDelete - " . $exc->getMessage());
        }

        $response = $this->client->getResponse();
        $content = $this->client->getResponse()->getContent();
        $json_content = json_decode($content, true);
        switch ($response->getStatusCode()) {
            case 200: //\Symfony\Component\HttpFoundation\Response::HTTP_OK:
                $this->message = 'The tag have been removed from the contact record.';
                $this->log('contactTagDelete - réponse OK');
                $this->log('contactTagDelete - FIN');
                return true;
                break;
            case 404://\Symfony\Component\HttpFoundation\Response::HTTP_NOT_FOUND:
                $this->message = 'Error, tag not found.';
                $this->log('contactTagDelete - réponse 404');
                $this->log('contactTagDelete - FIN');
                return true;
                break;
            //Si l'accès à l'API nous est refusé parceque la version est supérieur ou égale à 1.1 et 
            //que la clé n'est pas valide
            case 403://\Symfony\Component\HttpFoundation\Response::HTTP_FORBIDDEN:
                $this->message = 'Access denied to the FazaCRM\'s API, please check your private key.';
                $this->log('contactTagDelete - réponse HTTP FORBIDEN');
                return false;
                break;
            default:
                $this->message = 'Unknown error occured during the removal tag of this contact record.';
                $this->logError("contactTagDelete");
                return false;
                break;
        }
    }

    /**
     * Envois un mail lorsque l'erreur n'est pas traitée par le programme
     * @param string $context
     */
    public function logError($context) {
        /* @var $client \Zend\Http\Client */
        $client = $this->client;

        $this->log($context . ' - Status Code non géré : ' . $client->getResponse()->getStatusCode());
        $this->log($context . ' - URI : ' . $client->getUri());
        $this->log($context . ' - last request : ' . $client->getLastRawRequest());
        $this->log($context . ' - last response : ' . $client->getLastRawResponse());

        mail('urgence@fazae.com', 'erreur fazaCRMRESTClient', $context . ' - Status Code non géré : ' . $client->getResponse()->getStatusCode() . "\n" .
                $context . ' - URI : ' . $client->getUri() . "\n" .
                $context . ' - last request : ' . $client->getLastRawRequest() . "\n" .
                $context . ' - last response : ' . $client->getLastRawResponse() . "\n"
        );
    }

}
